@@ -65,12 +65,15 @@ class Placeholder:
         
         def literal(self, s):
                 return repr(s)
+                
+        def format_environment(self, s):
+                return s
 
         def re_environment(self, m):
                 if m.group(1) or not m.group(2) in os.environ:
                         return '$' + m.group(2)
                 else:
-                        return os.environ[m.group(2)]
+                        return self.format_environment(os.environ[m.group(2)])
 
         def expand_environment(self, text):
                 if not text:
@@ -106,6 +109,9 @@ class Placeholder:
                 
         # Do something on beginning this placeholder
         def enter(self):
+                if not self.begin or self.begin.get_deleted():
+                        return
+
                 self.buf.move_mark(self.buf.get_insert(), self.begin_iter())
 
                 if self.end:
@@ -305,7 +311,10 @@ class PlaceholderExpand(Placeholder):
                 self.timeout_id = None
                 
                 return False
-                
+        
+        def format_environment(self, text):
+                return self.literal(text)
+
         def substitute(self, text, formatter = None):
                 formatter = formatter or self.literal
 
@@ -404,7 +413,10 @@ class PlaceholderShell(PlaceholderExpand):
 
                 self.process_close()
                 return False
-                
+        
+        def literal(self, text):
+                return '"' + re.sub('[\\\\"]', '\\\\\\1', text) + '"'
+        
         def expand(self, text):
                 self.remove_timeout()
 
@@ -462,7 +474,7 @@ class PlaceholderEval(PlaceholderExpand):
                 if refs:
                         for ref in refs:
                                 self.refs.append(int(ref.strip()))
-        
+
         def get_mirrors(self, placeholders):
                 mirrors = PlaceholderExpand.get_mirrors(self, placeholders)
                 
@@ -617,7 +629,7 @@ class PlaceholderRegex(PlaceholderExpand):
                 # Try to compile pattern
                 try:
                         regex = re.compile(pattern, self.modifiers)
-                except re.Error, message:
+                except re.error, message:
                         sys.stderr.write('Could not compile regular expression: %s\n%s\n' % (pattern, message))
                         return False
                 
